'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function getContext(store, ns) {
    if (store._modules !== undefined) {
        var module_1 = store._modulesNamespaceMap[ns] || store._modules.root;
        return module_1.context;
    }
    return store;
}
function getterDescriptor(fn) {
    return {
        configurable: false,
        enumerable: false,
        get: fn
    };
}
function valueDescriptor(value) {
    return {
        configurable: false,
        enumerable: false,
        writable: false,
        value: value
    };
}
function Context() {
    var cache = new WeakMap();
    return {
        InstanceType: undefined,
        getGetters: function (getters) {
            return getters;
        },
        getInstance: function (store, ns) {
            if (ns === void 0) { ns = ''; }
            var fixedNs = ns.length ? ns + '/' : '';
            var context = getContext(store, fixedNs);
            var cachedInstance = cache.get(context.dispatch);
            if (cachedInstance) {
                return cachedInstance;
            }
            var commit = new Proxy(Object.create(null), {
                get: function (target, type) {
                    return (target[type] ||
                        (target[type] = function (payload, options) {
                            return context.commit(type, payload, options);
                        }));
                }
            });
            var dispatch = new Proxy(Object.create(null), {
                get: function (target, type) {
                    return (target[type] ||
                        (target[type] = function (payload, options) {
                            return context.dispatch(type, payload, options);
                        }));
                }
            });
            var instance = Object.create(null, {
                state: getterDescriptor(function () { return context.state; }),
                getters: getterDescriptor(function () { return context.getters; }),
                commit: valueDescriptor(commit),
                dispatch: valueDescriptor(dispatch)
            });
            cache.set(context.dispatch, instance);
            return instance;
        }
    };
}
Context.fromModule = function (module) {
    return Context();
};

exports.Context = Context;
