import { ActionContext, ActionTree, CommitOptions, DispatchOptions, GetterTree, MutationTree, Store } from "vuex";
declare type UnionToIntersection<U> = (U extends any ? (k: U) => void : never) extends ((k: infer I) => void) ? I : never;
declare type InferPayload<T> = T extends (_: any, payload?: any, options?: any) => any ? Parameters<T>[1] : never;
declare type Mutation<T> = T extends undefined ? (() => void) & ((payload: undefined, options?: CommitOptions) => void) : (payload: T, options?: CommitOptions) => void;
declare type InferMutation<T> = UnionToIntersection<Mutation<InferPayload<T>>>;
declare type InferMutations<T> = {
    [Key in keyof T]: InferMutation<T[Key]>;
};
declare type Action<T, R> = T extends undefined ? (() => R) & ((payload: undefined, options?: DispatchOptions) => R) : (payload: T, options?: DispatchOptions) => R;
declare type ActionReturn<T> = T extends (...args: any[]) => infer R ? R : void;
declare type InferAction<T> = UnionToIntersection<Action<InferPayload<T>, ActionReturn<T>>>;
declare type InferActions<T> = {
    [Key in keyof T]: InferAction<T[Key]>;
};
declare type InferGetters<T> = {
    [Key in keyof T]: T[Key] extends (state: any, getters: any) => infer R ? R : never;
};
declare type InferState<S> = S extends () => infer S ? S : S;
declare function Context<S extends (() => object) | object = {}, M extends MutationTree<any> = {}, A extends ActionTree<any, any> = {}, G extends GetterTree<any, any> = {}>(): {
    InstanceType: Readonly<{
        state: InferState<S>;
        commit: InferMutations<M>;
        dispatch: InferActions<A>;
        getters: InferGetters<G>;
    }>;
    getGetters(getters: any): InferGetters<G>;
    getInstance(store: ActionContext<any, any> | Store<any>, ns?: string): Readonly<{
        state: InferState<S>;
        commit: InferMutations<M>;
        dispatch: InferActions<A>;
        getters: InferGetters<G>;
    }>;
};
declare namespace Context {
    var fromModule: <S extends object | (() => object) = {}, M extends MutationTree<any> = {}, A extends ActionTree<any, any> = {}, G extends GetterTree<any, any> = {}>(module: Partial<{
        state: S;
        mutations: M;
        actions: A;
        getters: G;
    }>) => {
        InstanceType: Readonly<{
            state: InferState<S>;
            commit: InferMutations<M>;
            dispatch: InferActions<A>;
            getters: InferGetters<G>;
        }>;
        getGetters(getters: any): InferGetters<G>;
        getInstance(store: ActionContext<any, any> | Store<any>, ns?: string): Readonly<{
            state: InferState<S>;
            commit: InferMutations<M>;
            dispatch: InferActions<A>;
            getters: InferGetters<G>;
        }>;
    };
}
export default Context;
